close all
clearvars
clc

CT = PlotColor(6, 'blue');
set(groot,'defaultAxesColorOrder', CT);
set(groot,'defaultLineLineWidth', 2);
set(groot,'defaultAxesTitleFontWeight','normal');

saveflag    = 1;    % 1 to save results for each run

% Need to feed "par" containing parameter values and "MUs" containing
% containment policy and wage subsidies.

load HKK_parameters

PSIs = [0.0513,0.09,0.4:0.3:1.3];
npsi = length(PSIs);

for i = 1:8
    spec = i;
    par = par_ERT0;
    
    if i <= npsi
        par.psi = PSIs(i);
        if par.psi <= 1
            par.chi = PSIs(i);
        else
            par.chi = 1;
        end
        par.chi_bar = 0.0513;
        par.kappa = (par.chi - par.chi_bar)^2;
        par.chi_AB = 1;
    elseif i == 7
        par.chi = 1;
        par.chi_bar = par.chi;
        par.kappa = 0;
        par.chi_AB = 1;
    elseif i == 8
        par = par_SIR0;
        par.chi = 1;
        par.chi_bar = par.chi;
        par.kappa = 0;
        par.chi_AB = 1;
    end
    % Solve for the imperfect information model
    if i <= npsi || i >= 7
        imperfect_Solve_Compact
    end
end

varlist = {'I'; 'S'; 'R'; 'D'; 'C'; 'H'; ...
           'cs'; 'ci_tilde'; ...
           'ns'; 'ni_tilde'; ...
           'U0'; 'pi_dt_tilde'};

% Packing all variables
for i = 1:length(varlist)
	eval([varlist{i} '_ALL = [];'])
	for j = 1:npsi
		eval([varlist{i} '_ALL = [' varlist{i} '_ALL, ' varlist{i} '_v' num2str(j) '];'])
	end
end


%% Plot and Compare the Results %%
% -- Fig 3 --
fig1 = figure;

subplot(2,3,1)
fh = plot([I_ALL]*100);
box off
set(gca, 'XLim', [0, 150])
title('Infected, I')
ylabel('% of Init. Pop.')
xlabel('Weeks')

subplot(2,3,2)
fh = plot(([cs_ALL] - par_v1.c_ss)./par_v1.c_ss*100);
box off
% set(gca, 'YLim', [-30 2])
set(gca, 'XLim', [0, 150])
ylabel('% Dev. from Init. SS')
xlabel('Weeks')
title('Susceptibles Consumption')

subplot(2,3,3)
fh = plot(([ci_tilde_ALL] - par_v1.c_ss)./par_v1.c_ss*100);
box off
set(gca, 'YLim', [-30 2])
set(gca, 'XLim', [0, 150])
ylabel('% Dev. from Init. SS')
xlabel('Weeks')
title('Identified Consumption')

subplot(2,3,4)
plot([pi_dt_tilde_ALL]*100*18/7);
box off
set(gca, 'XLim', [0, 150])
title('Mortality Rate')
ylabel('%')
xlabel('Weeks')

subplot(2,3,5)
fh = plot(([ns_ALL] - par_v1.n_ss)./par_v1.n_ss*100);
box off
% set(gca, 'YLim', [-30 2])
set(gca, 'XLim', [0, 150])
ylabel('% Dev. from Init. SS')
xlabel('Weeks')
title('Susceptibles Hours')

subplot(2,3,6)
fh = plot(([ni_tilde_ALL] - par_v1.n_ss)./par_v1.n_ss*100);
box off
set(gca, 'YLim', [-30 2])
set(gca, 'XLim', [0, 150])
ylabel('% Dev. from Init. SS')
xlabel('Weeks')
title('Identified Hours')

STR = LegendText(PSIs, '\psi');
lh = legend(STR);
set(lh, 'Location', 'SouthEast', 'EdgeColor', 'none', 'FontSize', 8)


% -- Fig 4 --
U0s = [U0_ALL];
U0s_noFQ_Med = U0s;

save SW_noFQ_Med U0s_noFQ_Med

fig2 = figure;
subplot(2,1,1)

catnames = reordercats(categorical(STR),STR);
fh = bar(catnames, U0s);
set(gca, 'YLim', [min(U0s)-2 max(U0s)])

grid on
% title(['Imperfect Information Household'])
ylabel('Social Welfare (U_{0})')
set(gca, 'FontSize', 11)


set(groot,'defaultAxesColorOrder', 'remove');
set(groot,'defaultLineLineWidth', 'remove')
set(groot,'defaultAxesTitleFontWeight','remove');



